'' +--------------------------------------------------------------------------+
'' | Cluso's Propeller Operating System - OS module: _CMD  --> _CMD.CMD       |
'' +--------------------------------------------------------------------------+
'' |  Authors:       (c)2011 by Kwabena W. Agyman (Kye) & J. Moxham (Dr_Acula)|
'' |  Modifications: (c)2012 "Cluso99" (Ray Rodrick)                          |
'' |  License:       MIT License - See end of file for terms of use           |
'' +--------------------------------------------------------------------------+
'' This OS module uses an included module to perform most of the housekeeping.
'' Each module is called from, and returns to, the prop binary "_CMD.CMD"
'' Compile with PropTool + "PreSpin 0.3" by Andy Schenk www.insonix.ch/propeller                             
'' Some OS modules are derived from: 
''      KyeDOS v3 (c) 2011 by Kwabena W. Agyman (Kye) & J. Moxham (Dr_Acula)
'' See the included code for history and acknowledgments.
''
'' RR20120317   This module does......

'#include "__MODULE.spin"        ' include the common code for OS modules 

DAT
_ModuleStr    byte      "=== CMD ===",0        'module's name (string)

PRI executeCommand(stringPointer)  

''ifnot(str.stringCompareCI(string("cmd"), stringPointer))   '<-- module's name

'   crlf                                                
    result := shellLine(string("SD:>"))                 ' wait for a command
    
    str.copy(result,@commandline)      ' save for later on if this is a command line with parameters eg "xmodem r filename.txt"
'================
    str.copy(@commandline,_hub#_pBuffer_A)              ' pass commandline to next program
'=================

    str.tokenizeString(@commandline)
    \runFilename(string("_"), @commandline, string(".CMD")) ' load/run file if found
    \runFilename(string(" "), @commandline, string(".BIN")) ' load/run file if found
    shellCommands(@commandline)                         ' try inbuilt commands
    PrintStringCR(string("Command not found, try help"))

PRI runFilename(stringptr1, stringptr2, stringptr3)
' concatenate prefix+command+suffix to form a filename
' if file exists, load and run, else return
' uses @lineoftext2
    str.copy(stringptr1, @lineoftext2)
    str.stringConcatenate(@lineoftext2, stringptr2)
    str.stringConcatenate(@lineoftext2, stringptr3)
    str.trimString(@lineoftext2)
    result := \fat.bootPartition(@lineoftext2)          ' if found, load and run module

PRI shellCommands(stringPointer)

  programClear(stringPointer)
  programEcho(stringPointer)

  programReboot(stringPointer)

  programChangeDirectory(stringPointer)
  programChangeAttributes(stringPointer)

  programMakeFile(stringPointer)
  programMakeDirectory(stringPointer)

  programMove(stringPointer)
  programRemove(stringPointer)
  programEra(stringPointer)                             ' same as programremove

  programToggleLineFeed(stringPointer)


'  programDate(stringPointer)                                      
'  programTime(stringPointer)

'  programRamSet(stringPointer)
'  programRamGet(stringPointer)

'  programSQW(stringPointer)
'  programOUT(stringPointer)

'  programPlayWavFile(stringPointer)
'  programTapeWavFile(stringPointer)

' abort string("Command Not Found! Try Help")
' crlf

PRI shellLine(prompt)

  PrintString(prompt)
  repeat
    result := GetKey            ' scan all inputs (keyboard/comport)
    PrintChar(result)           ' send to vga and to com port
    str.buildString(result)     ' build an input string
  while((result <> _carriageReturnCharacter) and (result <> _lineFeedCharacter) and (result <> _nullCharacter))

  return str.trimString(str.builtstring(true))

PRI programClear(stringPointer)
  ifnot(str.stringCompareCI(string("clear"), stringPointer))
    waitcnt(clkfreq/1000 * 500 + cnt)                     ' delay 500ms
    printstring(string(_clearScreenCharacter, _homeCursorCharacter))
    abort false

PRI programEcho(stringPointer)
  ifnot(str.stringCompareCI(string("echo"), stringPointer))
    printstring(str.trimString(stringPointer += 5))
    crlf
    abort false

PRI programReboot(stringPointer)

  ifnot(str.stringCompareCI(string("reboot"), stringPointer))
    reboot

PRI programChangeDirectory(stringPointer)
  ifnot(str.stringCompareCI(string("cd"), stringPointer))
    stringPointer := str.tokenizeString(0)
    fat.changeDirectory(stringPointer)
    taf.changeDirectory(stringPointer)
    abort 0

PRI programChangeAttributes(stringPointer)
  ifnot(str.stringCompareCI(string("chmod"), stringPointer))
    fat.changeAttributes(str.tokenizeString(0), str.tokenizeString(0))
    abort 0

PRI programMove(stringPointer)
  ifnot(str.stringCompareCI(string("mv"), stringPointer))
    fat.moveEntry(str.tokenizeString(0), str.tokenizeString(0))
    abort 0

PRI programRemove(stringPointer)
  ifnot(str.stringCompareCI(string("rm"), stringPointer)) 
    fat.deleteEntry(str.tokenizeString(0))
    abort 0

PRI programERA(stringPointer) ' same as ProgramRemove
  ifnot(str.stringCompareCI(string("era"), stringPointer)) 
    fat.deleteEntry(str.tokenizeString(0))
    abort 0

PRI programMakeFile(stringPointer)
  ifnot(str.stringCompareCI(string("mkfile"), stringPointer))
    fat.newFile(str.tokenizeString(0))
    abort 0

PRI programMakeDirectory(stringPointer)
  ifnot(str.stringCompareCI(string("mkdir"), stringPointer))
    fat.newDirectory(str.tokenizeString(0))
    abort 0

PRI ProgramToggleLineFeed(stringPointer)
  ifnot(str.stringCompareCI(string("togglelf"), stringPointer))
    byte[_hub#_OS_Rows] ^= _hub#_LF_MASK                ' toggle LF mode (xor)
    abort 0


dat                                                   
{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |                                                            
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}